<?php

defined('SMF') or die('Restricted access');

/**
 * Prints synchronization data to be readed by BoWoB server.
 * @return void.
 */
function bowob_server_sync()
{
  header('Cache-Control: no-cache, must-revalidate');
  header('Content-Type: text/plain; charset=utf-8');

  include_once(dirname(realpath(__FILE__)) . SMF_DS . 'bowob_api.php');

  echo bowob_api_get_sync_data(
    request_int('id', -1),
    request_var('sync', ''),
    request_var('name', '') == '1',
    request_var('avatar', '') == '1',
    request_var('friends', '') == '1'
  );
}

/**
 * Creates a synchronization record and prints record identifiers to be readed by BoWoB client.
 * @return void.
 */
function bowob_client_sync()
{
  header('Cache-Control: no-cache, must-revalidate');
  header('Content-Type: text/plain');

  include_once(dirname(realpath(__FILE__)) . SMF_DS . 'bowob_api.php');

  echo bowob_api_new_sync(
    request_var('nick', ''),
    request_var('name', '') == '1',
    request_var('avatar', '') == '1'
  );
}

/**
 * Redirects to login page.
 * @return void.
 */
function bowob_redirect_login()
{
  global $boardurl;

  header('Location: ' . $boardurl . '/index.php?action=login');
  exit();
}

/**
 * Redirects to user profile page.
 * @return void.
 */
function bowob_redirect_profile()
{
  global $boardurl;

  $user_id = request_int('id', '-1');

  if ($user_id > 0)
    $link = $boardurl . '/index.php?action=profile;u='.$user_id;
  else
    $link = $boardurl;

  header('Location: ' . $link);
  exit();
}

/**
 * Gets BoWoB HTML code for show the chat.
 * @return string The HTML code.
 */
function bowob_code($app_id, $server_address)
{
  include_once(dirname(realpath(__FILE__)) . '/' . 'bowob_api.php');

  return bowob_api_get_code($app_id, $server_address);
}

/**
 * Checks if current user is logued.
 * @return boolean User is logued.
 */
function bowob_is_user_logued()
{
  global $context;

  return (boolean)$context['user']['is_logged'];
}

/**
 * Gets current user id.
 * @return int User id.
 */
function bowob_get_user_id()
{
  global $context;

  return $context['user']['id'];
}

/**
 * Gets current user nick.
 * @return string User nick.
 */
function bowob_get_user_nick()
{
  global $context;

  return $context['user']['username'];
}

/**
 * Gets current user name.
 * @return string User name.
 */
function bowob_get_user_name()
{
  global $context;

  return $context['user']['name'];
}

/**
 * Gets current user avatar url.
 * @return string User avatar.
 */
function bowob_get_user_avatar()
{
  if (bowob_is_user_logued())
  {
    global $boardurl, $context;

    $user_image = $context['user']['avatar']['href'];

    return $user_image;
  }

  return '';
}

/**
 * Gets current user friends.
 * @param int $id User id.
 * @param string $separator Separator between nicks.
 * @return string User friends.
 */
function bowob_get_user_friends($id, $separator)
{
  global $smcFunc;

  if($id <= 0)
  {
    return '';
  }

  $output = '';

  // Get buddy list from user (comma-separated ID list)
  $sql = "SELECT id_member, member_name, buddy_list
          FROM " . TABLE_PREFIX . "members
          WHERE id_member = '".(int)$id."'
          ";
  $friends = $smcFunc['db_fetch_assoc']($smcFunc['db_query']('', $sql, array())); 

  $friends=explode(',', $friends['buddy_list']);

  // Build a single SQL query to fetch member_name of all buddies from user buddy_list
  $sql = "SELECT member_name FROM " . TABLE_PREFIX . "members WHERE ";

  foreach ($friends as $k => $friend)
  {
    if ($friend)
    {
      if ($k > 0) $sql .= " OR";

      $sql .= " id_member = ".$friend;
    }
  }

  $request = $smcFunc['db_query']('', $sql, array());

  while ($friend = $smcFunc['db_fetch_assoc']($request))
    $output .= $friend['member_name'] .$separator;

  return $output;
}

/**
 * Stores a synchronization record in database.
 * @param string $auth Record auth.
 * @param int $creation Record creation time.
 * @param int $user_id Record user id.
 * @param string $user_nick Record user nick.
 * @param string $user_name Record user name.
 * @param string $user_avatar Record user avatar.
 * @param int $user_type Record user type.
 * @return int Record id.
 */
function bowob_store_sync($auth, $creation, $user_id, $user_nick, $user_name, $user_avatar, $user_type)
{
  global $smcFunc;

  $sql = "INSERT INTO " . TABLE_PREFIX . "bowob
	  (auth, creation, user_id, user_nick, user_name, user_avatar, user_type)
	  VALUES ('$auth', ".(int)$creation.", ".(int)$user_id.", '$user_nick', '$user_name', '$user_avatar', '$user_type')
	 ";

  $smcFunc['db_query']('', $sql, array());

  return $smcFunc['db_insert_id'](TABLE_PREFIX . 'bowob', 'id');
}

/**
 * Extracts a synchronization record from database.
 * @param int $id Record id.
 * @param string $auth Record auth.
 * @param int $expiration Record expiration time.
 * @return array Record values.
 */
function bowob_extract_sync($id, $auth, $expiration)
{
  global $smcFunc;

  $sql = "DELETE FROM " . TABLE_PREFIX . "bowob WHERE creation < " . (int)$expiration;

  $smcFunc['db_query']('', $sql, array());

  $sql = "SELECT auth, user_id, user_nick, user_name, user_avatar, user_type
	  FROM " . TABLE_PREFIX . "bowob
	  WHERE id = '".(int)$id."'
	  ";

  $rs = $smcFunc['db_fetch_assoc']($smcFunc['db_query']('', $sql, array()));

  if(!$rs || $rs['auth'] != $auth)
  {
    return array();
  }
  else
  {
    $sql = "DELETE FROM " . TABLE_PREFIX . "bowob WHERE id = " . (int)$id;

    $smcFunc['db_query']('', $sql, array());

    return array(
      'user_id' => $rs['user_id'],
      'user_nick' => $rs['user_nick'],
      'user_name' => $rs['user_name'],
      'user_avatar' => $rs['user_avatar'],
      'user_type' => $rs['user_type'],
    );
  }
}

/**
 * Gets filtered integer from $_REQUEST array
 * @param string $var Variable. 
 * @param string $default Default value if $var is empty.
 * @return int bowob_type.
 */
function request_int($var, $default)
{
  if ((int)$_REQUEST[$var] > 0)
    return $_REQUEST[$var];
  else
    return (int)$default;
}

/**
 * Gets filtered variable from $_REQUEST array
 * @param string $var Variable.
 * @param string $default Default value if $var is empty.
 * @return string Filtered variable value.
 */
function request_var($var, $default)
{
  if (!empty($_REQUEST[$var]))
    return $_REQUEST[$var];
  else
    return $default;
}

/**
 * Creates the BoWoB table in database.
 * @return void.
 */
function bowob_create_table()
{
  global $smcFunc;

  $sql = 'CREATE TABLE ' . TABLE_PREFIX. 'bowob (
          id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
          auth VARCHAR(50) NOT NULL DEFAULT \'\',
          creation INT(10) UNSIGNED NOT NULL DEFAULT \'0\',
          user_id INT(10) UNSIGNED NOT NULL DEFAULT \'0\',
          user_nick VARCHAR(50) NOT NULL DEFAULT \'\',
          user_name VARCHAR(50) NOT NULL DEFAULT \'\',
          user_avatar VARCHAR(200) NOT NULL DEFAULT \'\',
          user_type INT(10) UNSIGNED NOT NULL DEFAULT \'0\',
          PRIMARY KEY  (id)
      	) ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=utf8';

  $smcFunc['db_query']('', $sql, array());
}

/**
 * Drops the BoWoB table from database.
 * @return void.
 */
function bowob_drop_table()
{
  global $smcFunc;

  $sql = 'DROP TABLE ' . TABLE_PREFIX. 'bowob';

  $smcFunc['db_query']('', $sql, array());
}
